/*****************************************************************\

	Library of windows IME relative functions which part of xlib.c
	Jan-01	Genzoo.A Created.

\******************************************************************/
#include <windows.h>
#include <imm.h>
#include "xlibime.h"
#define MAX_IT(current, other)   if ((other) > (current)) (current) = (other)

static FARPROC pImmGetContext;
static FARPROC pImmReleaseContext;
static FARPROC pImmSetCompositionFontA;
static FARPROC pImmSetCompositionWindow;
HFONT  hImeFont = NULL;
static HANDLE _hImeLib = NULL;

static int cursor_pos_x = 0;
static int cursor_pos_y = 0;

void SetCurPos(const int x, const int y);

void SetCurPos(const int x, const int y)
{
  cursor_pos_x = x;
  cursor_pos_y = y;
}

/* there is no need direct call */
HANDLE  _IME_GetHandle()
{
  if (_hImeLib != NULL) return _hImeLib;

  _hImeLib = LoadLibrary("IMM32.DLL");
  if (_hImeLib == NULL){
    _hImeLib = INVALID_HANDLE_VALUE;
    return _hImeLib;
  }

#define IME_LOAD_LIB(__symbol__) \
  p ## __symbol__ = GetProcAddress(_hImeLib, #__symbol__); \
  if(!p ## __symbol__){ \
    FreeLibrary(_hImeLib); \
    _hImeLib = INVALID_HANDLE_VALUE; \
    return _hImeLib; \
  }

  IME_LOAD_LIB(ImmGetContext);
  IME_LOAD_LIB(ImmReleaseContext);
  IME_LOAD_LIB(ImmSetCompositionFontA);
  IME_LOAD_LIB(ImmSetCompositionWindow);

#undef IME_LOAD_LIB

  return _hImeLib;
}

void IME_Release(HWND hWnd)
{
  if(_hImeLib == INVALID_HANDLE_VALUE || !_hImeLib) return;
  IME_SetCompWinPos(hWnd,IME_SCWP_DEFAULT);
  FreeLibrary(_hImeLib);
  _hImeLib = NULL;
  hImeFont = NULL;
}

void IME_SetCompWinPos(HWND hWnd, ime_scwp_t ind)
{
  HIMC  hIMC;
  COMPOSITIONFORM cf;
  static BOOL isSetFont = FALSE;
  HANDLE hImeLib = _IME_GetHandle();
  if(hImeLib == INVALID_HANDLE_VALUE) return;
  hIMC = (*pImmGetContext)(hWnd);
  if (ind  == IME_SCWP_CURSOR){
    cf.dwStyle = CFS_POINT;
    cf.ptCurrentPos.x = cursor_pos_x;
    cf.ptCurrentPos.y = cursor_pos_y;
    MAX_IT(cf.ptCurrentPos.y,0);
  }
  else
    cf.dwStyle = CFS_DEFAULT;
  (*pImmSetCompositionWindow)(hIMC,&cf);
  (*pImmReleaseContext)(hWnd,hIMC);

  if(!isSetFont) isSetFont = _IME_SetCompWinFont(hWnd);
}

/* there is no need direct call */
BOOL _IME_SetCompWinFont(HWND hWnd)
{
  HIMC  hIMC;
  LOGFONT lf;
  HANDLE hImeLib = _IME_GetHandle();
  if(hImeLib == INVALID_HANDLE_VALUE || !hImeFont) return FALSE;;
  hIMC = (*pImmGetContext)(hWnd);
  GetObject(hImeFont, sizeof(LOGFONT), &lf);
  (*pImmSetCompositionFontA)(hIMC,&lf);
  (*pImmReleaseContext)(hWnd,hIMC);
  return TRUE;
}


